# -*- coding: utf-8 -*-
# Part of Odoo. See LICENSE file for full copyright and licensing details.

from odoo.exceptions import AccessDenied

from odoo import api, models, fields, registry, SUPERUSER_ID


class HrEmployee(models.Model):
    _inherit = ['hr.employee']
    _description = 'Employee'

    is_head_of_department = fields.Boolean('Is Head of Department')
    deputy_ids = fields.One2many('hr.employee.deputy', 'employee_id')

    def get_manager_of_department(self):
        if self.parent_id:
            return self.parent_id
        else:
            return False

    def get_head_of_department(self):
        self.ensure_one()
        if self.is_head_of_department:
            return self
        elif self.parent_id:
            return self.parent_id.get_head_of_department()
        else:
            return self.parent_id

    def get_deputies(self, model):
        self.ensure_one()
        deputy = self.deputy_ids.sudo().filtered(lambda activity: activity.module_id.model == model).deputy_id
        return deputy


class HrEmployeePublic(models.Model):
    _inherit = 'hr.employee.public'

    is_head_of_department = fields.Boolean('Is Head of Department')
    # code = fields.Char(string='Code')

