# Copyright 2017 ForgeFlow S.L. (https://www.forgeflow.com)
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class TierReview(models.Model):
    _inherit = "tier.review"
    _description = "Tier Review"

    @api.depends("reviewer_ids")
    def _compute_todo_by(self):
        """Show by group or by abbrev list of names"""
        num_show = 3  # Max number of users to display
        for rec in self:
            todo_by = False
            if rec.reviewer_group_id and rec.definition_id and not rec.definition_id.get_head_of_department and not rec.definition_id.get_manager_of_department:
                todo_by = _("Group %s") % rec.reviewer_group_id.name
            else:
                todo_by = ", ".join(rec.reviewer_ids[:num_show].mapped("display_name"))
                num_users = len(rec.reviewer_ids)
                if num_users > num_show:
                    todo_by = "{} (and {} more)".format(todo_by, num_users - num_show)
            rec.todo_by = todo_by

    def _get_reviewers(self):
        if self.definition_id and self.definition_id.get_head_of_department:
            main_request = self.env[self.model].browse(self.res_id)
            created_employee = main_request.create_uid.employee_id
            if created_employee:
                head = created_employee.get_head_of_department()
                if head:
                    deputy = head.get_deputies(self.model)
                    if deputy:
                        return head.user_id + deputy.user_id
                    else:
                        return head.user_id
                else:
                    raise ValidationError(f'Employee {str(created_employee.name)} do not have Head of function')
            else:
                raise ValidationError(f'Employee of User ({main_request.create_uid.login}) does not exist')

        elif self.definition_id and self.definition_id.get_manager_of_department:
            request = self.env[self.model].browse(self.res_id)
            employee_created = request.create_uid.employee_id
            if employee_created:
                manager = employee_created.get_manager_of_department()
                if manager:
                    return manager.user_id
                else:
                    raise ValidationError(_("The Employee - %s has not Manager") % employee_created.name)
            else:
                raise ValidationError(f'Employee of User ({request.create_uid.login}) does not exist')

        return super(TierReview, self)._get_reviewers()